@@ -44,8 +44,14 @@ Any executor inherited from :class:`BaseExecutor` always has the following **met
         the working directory, for persisting the artifacts of the executor. An artifact is a file or collection of files
         used during a workflow run.
 
+        By default it is not set, if you expect your executor to be persisted or to persist any data, remember to set it
+        to the desired value.
+
+        When a `BaseExecutor` is a component of a `CompoundExecutor`, its `workspace` value will be overriden by the `workspace`
+        coming from the `CompoundExecutor` unless a particular `workspace` value is set for the component `BaseExecutor`.
+
         :type: str
-        :default: environment variable :confval:`JINA_EXECUTOR_WORKDIR`, if not set then using current working dir, aka ``cwd``.
+        :default: None
 
     .. confval:: name
 
@@ -68,31 +74,72 @@ Any executor inherited from :class:`BaseExecutor` always has the following **met
         using :func:`BaseExecutor.load_config` from a YAML file. If a relative path is given then the root path is set to
         the path of the current YAML file.
 
+        Example of ``py_module`` usage:
+
+        1. This is a valid structure and it is RECOMMENDED:
+            - "my_cust_module" is a python module
+            - all core logic of your customized executor goes to ``__init__.py``
+            - to import ``foo.py``, you can use relative import, e.g. ``from .foo import bar``
+            - ``helper.py`` needs to be put BEFORE `__init__.py` in YAML ``py_modules``
+
+        This is also the structure given by ``jina hub new`` CLI.
+
+            .. highlight:: text
+            .. code-block:: text
+
+                 my_cust_module
+                   |- __init__.py
+                   |- helper.py
+                   |- config.yml
+                        |- py_modules
+                               |- helper.py
+                               |- __init__.py
+
+        2. This is a valid structure but not recommended:
+            - "my_cust_module" is not a python module (lack of __init__.py under the root)
+            - to import ``foo.py``, you must to use ``from jinahub.foo import bar``
+            - ``jinahub`` is a common namespace for all plugin-modules, not changeable.
+            - ``helper.py`` needs to be put BEFORE `my_cust.py` in YAML ``py_modules``
+
+            .. highlight:: text
+            .. code-block:: text
+
+                 my_cust_module
+                   |- my_cust.py
+                   |- helper.py
+                   |- config.yml
+                        |- py_modules
+                               |- helper.py
+                               |- my_cust.py
+
         :type: str/List[str]
         :default: ``None``
 
     .. confval:: pea_id
 
-        the integer index used for distinguish each parallel pea of this executor, useful in :attr:`pea_workspace`
+        the integer index used for distinguish each parallel pea of this executor, required in :attr:`shard_workspace`
 
         :type: int
         :default: ``'${{root.metas.pea_id}}'``
 
-    .. confval:: separated_workspace
+    .. confval:: root_workspace
 
-        whether to isolate the data of the parallel of this executor. If ``True``, then each parallel pea works in its own
-        workspace specified in :attr:`pea_workspace`
+        the workspace of the root executor. It will be the same as `executor` except in the case when an `Executor` inside a `CompoundExecutor` is used,
+        or when a `BaseNumpyIndexer` is used with a `ref_indexer`.
 
-        :type: bool
-        :default: ``'${{root.metas.separated_workspace}}'``
-        
-    .. confval:: pea_workspace
-
-        the workspace of each parallel pea, useful when :attr:`separated_workspace` is set to True. All data and IO operations
-        related to this parallel pea will be conducted under this workspace. It is often set as the sub-directory of :attr:`workspace`.
+        By default, jina will try to find if a `dump` of the executor can be found in `workspace`, otherwise it will try to find it under `root_workspace`
+        assuming it may be part of a `CompoundExecutor`.
 
         :type: str
-        :default: ``'${{root.metas.workspace}}/${{root.metas.name}}-${{root.metas.pea_id}}'``
+        :default: ``'${{root.metas.workspace}}'``
+
+    .. confval:: root_name
+
+        the name of the root executor. It will be the same as `executor` except in the case when an `Executor` inside a `CompoundExecutor` is used,
+        or when a `BaseNumpyIndexer` is used with a `ref_indexer`
+
+        :type: str
+        :default: ``'${{root.metas.name}}'``
 
     .. confval:: read_only
 
@@ -106,7 +153,7 @@ Any executor inherited from :class:`BaseExecutor` always has the following **met
 
     .. note::
 
-        ``separated_workspace``, ``pea_workspace`` and ``pea_id`` is set in a way that when the executor ``A`` is used as
+        ``pea_id`` is set in a way that when the executor ``A`` is used as
         a component of a :class:`jina.executors.compound.CompoundExecutor` ``B``, then ``A``'s setting will be overrided by B's counterpart.
 
     These **meta** fields can be accessed via `self.is_trained` or loaded from a YAML config via :func:`load_config`:
