@@ -10,8 +10,7 @@ import threading
 import uuid
 from collections import OrderedDict, defaultdict
 from contextlib import ExitStack
-from typing import Optional, Union, Tuple, List, Set, Dict, TextIO, TypeVar
-from urllib.request import Request, urlopen
+from typing import Optional, Union, Tuple, List, Set, Dict, TextIO
 
 from .builder import build_required, _build_flow, _optimize_flow, _hanging_pods
 from .. import __default_host__
@@ -19,17 +18,15 @@ from ..clients import Client, WebSocketClient
 from ..enums import FlowBuildLevel, PodRoleType, FlowInspectType
 from ..excepts import FlowTopologyError, FlowMissingPodError
 from ..helper import colored, \
-    get_public_ip, get_internal_ip, typename, ArgNamespace, random_identity
+    get_public_ip, get_internal_ip, typename, ArgNamespace, download_mermaid_url
 from ..jaml import JAML, JAMLCompatible
 from ..logging import JinaLogger
 from ..parsers import set_client_cli_parser, set_gateway_parser, set_pod_parser
 
-__all__ = ['BaseFlow', 'FlowLike']
+__all__ = ['BaseFlow']
 
 from ..peapods import BasePod
 
-FlowLike = TypeVar('FlowLike', bound='BaseFlow')
-
 
 class FlowType(type(ExitStack), type(JAMLCompatible)):
     pass
@@ -76,7 +73,6 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         self._last_changed_pod = ['gateway']  #: default first pod is gateway, will add when build()
         self._update_args(args, **kwargs)
         self._env = env  #: environment vars shared by all pods in the flow
-        self._identity = None
         if isinstance(self.args, argparse.Namespace):
             self.logger = JinaLogger(self.__class__.__name__, **vars(self.args))
         else:
@@ -108,7 +104,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
             else:
                 endpoint = []
 
-        if isinstance(endpoint, list) or isinstance(endpoint, tuple):
+        if isinstance(endpoint, (list, tuple)):
             for idx, s in enumerate(endpoint):
                 if s == pod_name:
                     raise FlowTopologyError('the income/output of a pod can not be itself')
@@ -152,7 +148,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
                            read_only=True,
                            runtime_cls='GRPCRuntime',
                            pod_role=PodRoleType.GATEWAY,
-                           identity=self._identity or random_identity()
+                           identity=self.args.identity
                            ))
 
         kwargs.update(self._common_kwargs)
@@ -161,7 +157,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         self._pod_nodes[pod_name] = BasePod(args, needs)
 
     def needs(self, needs: Union[Tuple[str], List[str]],
-              name: str = 'joiner', *args, **kwargs) -> FlowLike:
+              name: str = 'joiner', *args, **kwargs) -> 'BaseFlow':
         """
         Add a blocker to the flow, wait until all peas defined in **needs** completed.
 
@@ -173,10 +169,10 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
             raise FlowTopologyError('no need to wait for a single service, need len(needs) > 1')
         return self.add(name=name, needs=needs, pod_role=PodRoleType.JOIN, *args, **kwargs)
 
-    def needs_all(self, name: str = 'joiner', *args, **kwargs) -> FlowLike:
+    def needs_all(self, name: str = 'joiner', *args, **kwargs) -> 'BaseFlow':
         """
         Collect all hanging Pod so far and add a blocker to the flow, wait until all handing peas completed.
-        :param copy_flow: when set to true, then always copy the current flow and do the modification on top of it then return, otherwise, do in-line modification
+        
         :param name: the name of this joiner, by default is ``joiner``
         :return: the modified flow
         """
@@ -190,7 +186,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
             needs: Union[str, Tuple[str], List[str]] = None,
             copy_flow: bool = True,
             pod_role: 'PodRoleType' = PodRoleType.POD,
-            **kwargs) -> FlowLike:
+            **kwargs) -> 'BaseFlow':
         """
         Add a pod to the current flow object and return the new modified flow object.
         The attribute of the pod can be later changed with :py:meth:`set` or deleted with :py:meth:`remove`
@@ -257,7 +253,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
 
         return op_flow
 
-    def inspect(self, name: str = 'inspect', *args, **kwargs) -> FlowLike:
+    def inspect(self, name: str = 'inspect', *args, **kwargs) -> 'BaseFlow':
         """Add an inspection on the last changed Pod in the Flow
 
         Internally, it adds two pods to the flow. But no worry, the overhead is minimized and you
@@ -328,7 +324,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
             # no inspect node is in the graph, return the current graph
             return self
 
-    def build(self, copy_flow: bool = False) -> FlowLike:
+    def build(self, copy_flow: bool = False) -> 'BaseFlow':
         """
         Build the current flow and make it ready to use
 
@@ -432,11 +428,13 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
             for k, v in self._env.items():
                 os.environ[k] = str(v)
 
-        for v in self._pod_nodes.values():
-            self.enter_context(v)
-            if not v.is_ready:
-                self.logger.error(f'Pod {v!r} can not be started, Flow is aborted')
-                break
+        for k, v in self:
+            try:
+                self.enter_context(v)
+            except Exception as ex:
+                self.logger.error(f'{k}:{v!r} can not be started due to {ex!r}, Flow is aborted')
+                self.close()
+                raise
 
         self.logger.info(f'{self.num_pods} Pods (i.e. {self.num_peas} Peas) are running in this Flow')
 
@@ -454,7 +452,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         """Get the number of peas (parallel count) in this flow"""
         return sum(v.num_peas for v in self._pod_nodes.values())
 
-    def __eq__(self, other: FlowLike):
+    def __eq__(self, other: 'BaseFlow'):
         """
         Comparing the topology of a flow with another flow.
         Identification is defined by whether two flows share the same set of edges.
@@ -552,7 +550,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
              vertical_layout: bool = False,
              inline_display: bool = False,
              build: bool = True,
-             copy_flow: bool = False) -> FlowLike:
+             copy_flow: bool = False) -> 'BaseFlow':
         """
         Visualize the flow up to the current point
         If a file name is provided it will create a jpg image with that name,
@@ -605,7 +603,7 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
                 pass
 
         if output:
-            op_flow._download_mermaid_url(url, output)
+            download_mermaid_url(url, output)
         elif not showed:
             op_flow.logger.info(f'flow visualization: {url}')
 
@@ -628,20 +626,6 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
 
         return f'https://mermaid.ink/{img_type}/{encoded_str}'
 
-    def _download_mermaid_url(self, mermaid_url, output) -> None:
-        """
-        Rendering the current flow as a jpg image, this will call :py:meth:`to_mermaid` and it needs internet connection
-        :param path: the file path of the image
-        :param kwargs: keyword arguments of :py:meth:`to_mermaid`
-        :return:
-        """
-        try:
-            req = Request(mermaid_url, headers={'User-Agent': 'Mozilla/5.0'})
-            with open(output, 'wb') as fp:
-                fp.write(urlopen(req).read())
-        except:
-            self.logger.error('can not download image, please check your graph and the network connections')
-
     @build_required(FlowBuildLevel.GRAPH)
     def to_swarm_yaml(self, path: TextIO):
         """
@@ -752,6 +736,12 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         for k, p in self:
             if hasattr(p.args, 'workspace_id'):
                 p.args.workspace_id = value
+                for k, v in p.peas_args.items():
+                    if v and isinstance(v, argparse.Namespace):
+                        v.workspace_id = value
+                    if v and isinstance(v, List):
+                        for i in v:
+                            i.workspace_id = value
 
     @property
     def identity(self) -> Dict[str, str]:
@@ -765,33 +755,11 @@ class BaseFlow(JAMLCompatible, ExitStack, metaclass=FlowType):
         :param value: a hexadecimal UUID string
         """
         uuid.UUID(value)
+        self.args.identity = value
+        # Re-initiating logger with new identity
+        self.logger = JinaLogger(self.__class__.__name__, **vars(self.args))
         for _, p in self:
             p.args.identity = value
-        self._identity = value
-
-    def index(self):
-        raise NotImplementedError
-
-    def search(self):
-        raise NotImplementedError
-
-    def index_ndarray(self):
-        raise NotImplementedError
-
-    def index_lines(self):
-        raise NotImplementedError
-
-    def index_files(self):
-        raise NotImplementedError
-
-    def search_ndarray(self):
-        raise NotImplementedError
-
-    def search_lines(self):
-        raise NotImplementedError
-
-    def search_files(self):
-        raise NotImplementedError
 
     # for backward support
     join = needs
