@@ -6,10 +6,10 @@ def set_hub_pushpull_parser(parser=None):
     if not parser:
         parser = set_base_parser()
 
-    from .login import mixin_hub_login_parser
+    from .login import mixin_hub_docker_login_parser
     from .pushpull import mixin_hub_pushpull_parser
 
-    mixin_hub_login_parser(parser)
+    mixin_hub_docker_login_parser(parser)
     mixin_hub_pushpull_parser(parser)
     return parser
 
@@ -18,10 +18,10 @@ def set_hub_build_parser(parser=None):
     if not parser:
         parser = set_base_parser()
 
-    from .login import mixin_hub_login_parser
+    from .login import mixin_hub_docker_login_parser
     from .build import mixin_hub_build_parser
 
-    mixin_hub_login_parser(parser)
+    mixin_hub_docker_login_parser(parser)
     mixin_hub_build_parser(parser)
 
     return parser
@@ -31,10 +31,7 @@ def set_hub_list_parser(parser=None):
     if not parser:
         parser = set_base_parser()
 
-    from .login import mixin_hub_login_parser
     from .list import mixin_hub_list_parser
-
-    mixin_hub_login_parser(parser)
     mixin_hub_list_parser(parser)
 
     return parser
@@ -55,14 +52,12 @@ def set_hub_parser(parser=None):
         parser = set_base_parser()
 
     spp = parser.add_subparsers(dest='hub',
-                                description='use "%(prog)-8s [sub-command] --help" '
+                                description='use `%(prog)-8s [sub-command] --help` '
                                             'to get detailed information about each sub-command', required=True)
 
-    from .login import mixin_hub_login_parser
-    mixin_hub_login_parser(
-        spp.add_parser('login', help='login via Github to push images to Jina hub registry',
-                       description='Login via Github to push images to Jina hub registry',
-                       formatter_class=_chf))
+    spp.add_parser('login', help='login via Github to push images to Jina hub registry',
+                   description='Login via Github to push images to Jina hub registry',
+                   formatter_class=_chf)
 
     set_hub_new_parser(
         spp.add_parser('new', aliases=['init', 'create'], help='create a new Hub executor or app using cookiecutter',
