@@ -15,12 +15,12 @@ def mixin_zed_runtime_parser(parser):
                     help='''
 The config of the executor, it could be one of the followings: 
 - an Executor-level YAML file path (*.yml/yaml) 
-- a name of a class inherited from "jina.Executor"
-- a docker image (must start with "docker://")
-- builtin executors, e.g. "_pass", "_logforward", "_merge" 
-- the string literal of a YAML config (must start with "!")
+- a name of a class inherited from `jina.Executor`
+- a docker image (must start with `docker://`)
+- builtin executors, e.g. `_pass`, `_logforward`, `_merge` 
+- the string literal of a YAML config (must start with `!`)
 - the string literal of a JSON config
-- the string literal of a YAML driver config (must start with "- !!")
+- the string literal of a YAML driver config (must start with `- !!`)
 
 When use it under Python, one can use the following values additionally:
 - a Python dict that represents the config
